@@ -10,9 +10,10 @@ using namespace Rcpp;
 //' @param Fec_at_Age Vector of mature weight-at-age
 //' @param V_at_Age Vector of selectivity-at-age
 //' @param maxage Maximum age
-//' @param R0x R0 for this simulation
+//' @param R0x R0 for this simulation. Set = 1 if SRrelx = 3 for per-recruit calculations
 //' @param SRrelx SRR type for this simulation. Use 3 for per-recruit calculations, i.e. constant recruitment.
-//' @param hx numeric. Steepness value for this simulation
+//' @param hx numeric. Steepness value for this simulation. Not used if SRrelx = 3.
+//' @param SSBpR numeric. Unfished spawners per recruit for this simulation. Not used if SRrelx = 3.
 //' @param opt Option. 1 = return -Yield, 2= return all MSY calcs
 //' @param plusgroup Integer. Default = 0 = no plus-group. Use 1 to include a plus-group
 //' @return See `opt`
@@ -24,9 +25,10 @@ NumericVector MSYCalcs(double logF,
                   NumericVector Fec_at_Age,
                   NumericVector V_at_Age,
                   int maxage,
-                  double R0x,
-                  int SRrelx,
-                  double hx,
+                  double R0x = 1,
+                  int SRrelx = 3,
+                  double hx = 1,
+                  double SSBpR = 0,
                   int opt=1,
                   int plusgroup=1) {
   // Box 3.1 Walters & Martell 2004
@@ -61,24 +63,39 @@ NumericVector MSYCalcs(double logF,
   double SBF = sum(lx * Fec_at_Age);
   double B0 = sum(l0 * Wt_at_Age); // biomass-per-recruit
   double BF = sum(lx * Wt_at_Age);
+  
+  double N0 = sum(l0); // numbers per recruit
+  double SN0 = sum(l0 * Fec_at_Age); // spawning numbers per recruit
+  
+  double R0 = 0;  // The 'new' R0 associated with Egg0, reca, recb
+  double R20 = 0; // The recruitment at 20% of R0 with reca, recb, Egg0
+  double h = 1;   // The steepness associated with Egg0, new R0, reca, recb
 
   // double SPR = EggF/Egg0;
   double RelRec = 0;
   if (hx >0.999) hx = 0.999;
 
   if (SRrelx==1) {
-    double recK = (4*hx)/(1-hx); // Goodyear compensation ratio
-    double reca = recK/Egg0;
-    double recb = (reca*Egg0-1)/(R0x*Egg0);
+    double recK = (4*hx)/(1-hx); // Goodyear compensation ratio associated with SSBpR, R0x, and hx
+    double reca = recK/SSBpR;
+    double recb = (reca*SSBpR-1)/(R0x*SSBpR);
     RelRec = (reca*EggF-1)/(recb*EggF);
+    R0 = (reca*Egg0-1)/(recb*Egg0);
+    R20 = reca * 0.2 * Egg0 * R0/(1 + recb * 0.2 * Egg0 * R0);
+    h = R20/R0;
   }
   if (SRrelx==2) {
-    double bR = (log(5*hx)/(0.8*SB0));
-    double aR = exp(bR*SB0)/(SB0/R0x);
-    RelRec = (log(aR*EggF/R0x))/(bR*EggF/R0x);
+    double bR = log(5*hx)/(0.8*R0x*SSBpR);
+    double aR = exp(bR*R0x*SSBpR)/SSBpR;
+    RelRec = log(aR*EggF)/(bR*EggF);
+    R0 = log(aR*Egg0)/(bR*Egg0);
+    R20 = aR * 0.2 * Egg0 * R0 * exp(-bR * 0.2 * Egg0 * R0);
+    h = R20/R0;
   }
   if (SRrelx==3) {
     RelRec = R0x;
+    R0 = RelRec;
+    h = 1;
   }
   if (RelRec<0) RelRec = 0;
 
@@ -90,20 +107,25 @@ NumericVector MSYCalcs(double logF,
     out[0] = -Yield;
     return(out);
   } else {
+    
 
-    NumericVector out(11);
+    NumericVector out(15);
     out[0] = Yield;
     out[1] = FF;
     out[2] = SBF * RelRec;
-    out[3] = (SBF * RelRec)/(SB0 * R0x);
-    out[4] = (BF * RelRec)/(B0 * R0x);
+    out[3] = (SBF * RelRec)/(SB0 * R0);
+    out[4] = (BF * RelRec)/(B0 * R0);
     out[5] = BF * RelRec;
     out[6] = vBF * RelRec;
-    out[7] = (vBF * RelRec)/(vB0 * R0x);
-    out[8]=RelRec;
-    out[9] = SB0*R0x;
-    out[10] = B0*R0x;
-    out.names() = CharacterVector::create("Yield", "F", "SB", "SB_SB0", "B_B0", "B", "VB", "VB_VB0", "RelRec", "SB0", "B0");
+    out[7] = (vBF * RelRec)/(vB0 * R0);
+    out[8] = RelRec;
+    out[9] = SB0*R0;
+    out[10] = B0*R0;
+    out[11] = R0;
+    out[12] = h;
+    out[13] = N0 * R0;
+    out[14] = SN0 * R0;
+    out.names() = CharacterVector::create("Yield", "F", "SB", "SB_SB0", "B_B0", "B", "VB", "VB_VB0", "RelRec", "SB0", "B0", "R0", "h", "N0", "SN0");
 
     return(out);
   }
@@ -118,7 +140,6 @@ NumericMatrix Ref_int_cpp(NumericVector F_search,
               NumericVector Mat_at_Age,
               NumericVector Fec_at_Age,
               NumericVector V_at_Age,
-              int SRrelx,
               int maxage,
               int plusgroup=1) {
 
@@ -128,7 +149,7 @@ NumericMatrix Ref_int_cpp(NumericVector F_search,
   for (int i=0; i<ncol; i++) {
     double logF = log(F_search[i]);
     NumericVector msys = MSYCalcs(logF, M_at_Age, Wt_at_Age, Mat_at_Age, Fec_at_Age,
-                                      V_at_Age, maxage, 1, SRrelx, 1,2,plusgroup);
+                                  V_at_Age, maxage, 1, 3, 1, 0, 2, plusgroup);
     out(0,i) = msys[0];
     out(1,i) = msys[3];
     out(2,i) = msys[8]/msys[2];
